Module crypto_bigint

Module crypto_bigint 

Re-exports

pub use element::CryptoBigintElement;
pub use encoding::Error;

Modules

boxed_uint ๐Ÿ”’ alloc
c ๐Ÿ”’
composition ๐Ÿ”’
Composition of binary quadratic forms in constant time.
element ๐Ÿ”’
encoding ๐Ÿ”’
Compressed and uncompressed encoding of primitive reduced positive definite binary quadratic forms of negative discriminants (even or odd). While this is of greater scope than this library (which restricts itself to odd discriminants), it ensures the wire format allows the library to expand to include also implementing binary quadratic forms of even discriminants (without requiring a distinct encoding specification for those, when the time comes).
reduction ๐Ÿ”’
A constant-time reduction algorithm.
sqrt ๐Ÿ”’
uint ๐Ÿ”’

Type Aliases

I ๐Ÿ”’
A signed integer type.