pub(super) trait Limbs:
From<u8>
+ CtAssign
+ Zero
+ One
+ Limbs {
// Required methods
fn wide_div_rem_thin(
wide: Self::Wide,
thin: &NonZero<Self>,
) -> (Self::Wide, Self);
fn coprime(a: Self, b_abs: Self, c: Self::Wide) -> Choice;
}Expand description
The required view over a collection of limbs to validate a binary quadratic form.
Required Methods
Sourcefn wide_div_rem_thin(
wide: Self::Wide,
thin: &NonZero<Self>,
) -> (Self::Wide, Self)
fn wide_div_rem_thin( wide: Self::Wide, thin: &NonZero<Self>, ) -> (Self::Wide, Self)
Divide a wide value by a thin value, yielding the quotient and remainder.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".