I

Type Alias I 

pub(crate) type I<U> = (Choice, U);
A signed integer type.

The Choice represents if the number is positive. The other term represents the absolute value of the number. When the number is zero, whether or not the number is considered positive is undefined.