Cl15k

Struct Cl15k 

pub struct Cl15k<Up, Udk> {
    p: Odd<Up>,
    absolute_value: Udk,
}
A fundamental discriminant as part of the CL15 cryptosystem.

This is constructed as detailed in “Linearly Homomorphic Encryption from DDH” by Guilhem Castagnos and Fabien Laguillaumie (https://eprint.iacr.org/2025/047), corresponding to $\Delta_K$.

Up is the numeric type used to represent the prime p. Udk is the numeric type used to represent the discriminant’s absolute value, the product $q * p$.

Fields

p: Odd<Up>absolute_value: Udk

Implementations

Source

impl<Up, Udk> Cl15k<Up, Udk>

pub fn p(&self) -> &Up

The prime p from the setup.