Module primes
class_
groups
0.0.2-alpha
Module primes
Module Items
Enums
Functions
In crate class_
groups
class_groups
Module
primes
Enums

Error
๐
Functions

next_
prime
๐
Get the next prime greater than or equal to the seed.
next_
prime_
candidates
๐
Get the candidates for the next prime greater than or equal to the seed.