Compressed and uncompressed encoding of primitive reduced positive definite binary quadratic
forms of negative discriminants (even or odd). While this is of greater scope than this library
(which restricts itself to odd discriminants), it ensures the wire format allows the library to
expand to include also implementing binary quadratic forms of even discriminants (without
requiring a distinct encoding specification for those, when the time comes).