pub struct CryptoBigintElement<U: Limbs> {
a: U,
b: (Choice, U),
c: U::Wide,
discriminant_abs: U::Wide,
}Expand description
A constant-time primitive element of a class group, implemented via crypto-bigint.
This only supports discriminants delta such that $delta < 0, |delta| \cong 1 \mod 2$.
This is implemented in time variable to the discriminant yet constant to the a, b, c
coefficients. This prevents timing analysis from leaking the elements being composed. It is
only recommended for provers as it has a significant performance overhead compared to other
backends, which verifiers should take advantage of.
This implementation does not itself allocate and can be used with crypto_bigint::Uint to
achieve elements which are fixed size and live on the stack, albeit only with support for a
bounded subset of discriminants. Alternatively, crypto_bigint::BoxedUint may be used to
support all discriminants, at the cost of using the heap.
Fields
a: UThe a coefficient.
This may not be reduced but is bounded to less than the square root of the discriminant.
b: (Choice, U)The b coefficient.
This may not be reduced but is bounded to have an absolute value less than the square root of the discriminant.
c: U::WideThe c coefficient.
This may not be reduced but is bounded to be less than the discriminant.
discriminant_abs: U::WideThe absolute value of the negative discriminant for this form.
This is used to recalculate the c coefficient after composition.
Implementations
Sourceimpl<U: Limbs> CryptoBigintElement<U>
impl<U: Limbs> CryptoBigintElement<U>
Sourcefn identity_from_discriminant(discriminant_abs: U::Wide) -> Self
fn identity_from_discriminant(discriminant_abs: U::Wide) -> Self
This is only valid for forms of negative odd discriminant.
Sourceimpl<U: Limbs> CryptoBigintElement<U>
impl<U: Limbs> CryptoBigintElement<U>