pub(super) fn t(
a: NonZero<BoxedUint>,
b: BoxedUint,
) -> (Choice, NonZero<BoxedUint>)Available on crate feature
alloc only.Expand description
The implementation of the t function as described in the specification.
This function runs in time variable to the input. While a constant-time variant is feasible, as the binary GCD is itself feasible to implement in constant-time and we have a bound on termination, there’s not currently value to such an implementation when compression as a whole is currently posited in variable-time.