pub enum Cl15Error {
SmallP,
NoQ,
}Expand description
An error when sampling discriminants for the CL15 cryptosystem.
Variants
SmallP
The odd prime p was too small.
NoQ
There were no candidates for the odd prime q.
In effect, this means the fundamental discriminant was too small with regards to the
specified odd prime p.