Table

Struct Table 

pub struct Table<E: ElementExt>(usize, Vec<E>);
Available on crate feature alloc only.
A table to perform multiplications with.

Tuple Fields

0: usize1: Vec<E>

Implementations

Source

impl<E: ElementExt> Table<E>

pub fn new(bits: u32, identity: E, element: E) -> Self

Create a new table.

This function executes in constant-time w.r.t. element if double, add are constant-time.

pub fn new_for_scalar_bits(scalar_bits: usize, identity: E, element: E) -> Self

Create a new table of size optimal for a scalar-length.

This is usable in ad-hoc multiplications where creating the table, and performing the multiplication with it, should not cost more than performing the multiplication out-right.

pub fn bits(&self) -> usize

The bits preprocessed by this table.